@@ -1,9 +1,11 @@
 import abc
 import logging
 import os
+import re
 import shutil
 from abc import ABC
 from copy import copy, deepcopy
+from importlib import import_module
 from typing import Any, Dict, List, Union
 
 from .config import config as global_config
@@ -15,6 +17,21 @@ try:
 except Exception:
     V1alpha1S3Artifact = object
 
+field_regex = re.compile("^[a-zA-Z0-9][-a-zA-Z0-9]*$")
+field_errmsg = "name must consist of alpha-numeric characters or '-', and "\
+    "must start with an alpha-numeric character (e.g. My-name1-2, 123-NAME)"
+subdomain_regex = re.compile(
+    "^[a-z0-9]([-a-z0-9]*[a-z0-9])?(\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*$")
+subdomain_errmsg = "a lowercase RFC 1123 subdomain must consist of lower case"\
+    " alpha-numeric characters, '-' or '.', and must start and end with an "\
+    "alpha-numeric character (e.g. 'example.com')"
+param_regex = re.compile("^[-a-zA-Z0-9_]+[-a-zA-Z0-9_]*$")
+param_errmsg = "parameter/artifact name must consist of alpha-numeric "\
+    "characters, '_' or '-' e.g. my_param_1, MY-PARAM-1"
+key_regex = re.compile("^[a-z0-9][-a-z0-9]*$")
+key_errmsg = "key must consist of lower case alpha-numeric characters or '-',"\
+    "and must start with an alpha-numeric character (e.g. 'my-key')"
+
 
 class S3Artifact(V1alpha1S3Artifact):
     """
@@ -155,11 +172,30 @@ class LineageClient(ABC):
         pass
 
 
+def import_func(s):
+    fields = s.split(".")
+    if fields[0] == __name__ or fields[0] == "":
+        fields[0] = ""
+        mod = import_module(".".join(fields[:-1]), package=__name__)
+    else:
+        mod = import_module(".".join(fields[:-1]))
+    return getattr(mod, fields[-1])
+
+
 class CustomArtifact(ABC):
+    redirect = None
+
     @abc.abstractmethod
     def get_urn(self) -> str:
         pass
 
+    @staticmethod
+    def from_urn(urn):
+        protocol = urn.split("://")[0]
+        name = global_config["artifact_register"][protocol]
+        custom = import_func(name)
+        return custom.from_urn(urn)
+
     def __repr__(self):
         return self.get_urn()
 
