@@ -7,7 +7,7 @@ from typing import Any, Dict, List, Optional, Union
 
 import jsonpickle
 
-from .common import CustomArtifact, S3Artifact
+from .common import CustomArtifact, S3Artifact, param_errmsg, param_regex
 from .config import config
 from .utils import randstr, s3_config, upload_s3
 
@@ -87,6 +87,8 @@ class AutonamedDict(UserDict):
         super().__init__(*args)
 
     def __setitem__(self, key, value):
+        assert param_regex.match(key), "Invalid parameter/artifact name '%s':"\
+            " %s" % (key, param_errmsg)
         value.name = key
         value.step = self.step
         value.template = self.template
@@ -623,14 +625,17 @@ class InputArtifact(ArgoVar):
         from .task import Task
         if self.save_as_parameter:
             if self.name is not None:
+                suffix = "" if self._sub_path is None else \
+                    "/%s" % self._sub_path
                 if self.step is not None:
                     if isinstance(self.step, Task):
-                        return "{{tasks.%s.inputs.parameters.dflow_art_%s}}"\
-                            % (self.step.id, self.name)
+                        return "{{tasks.%s.inputs.parameters.dflow_art_%s}}%s"\
+                            % (self.step.id, self.name, suffix)
                     else:
-                        return "{{steps.%s.inputs.artifacts.dflow_art_%s}}"\
-                            % (self.step.id, self.name)
-                return "{{inputs.artifacts.dflow_art_%s}}" % self.name
+                        return "{{steps.%s.inputs.parameters.dflow_art_%s}}%s"\
+                            % (self.step.id, self.name, suffix)
+                return "{{inputs.parameters.dflow_art_%s}}%s" % (self.name,
+                                                                 suffix)
             return ""
         if self.name is not None:
             if self.step is not None:
@@ -645,6 +650,7 @@ class InputArtifact(ArgoVar):
 
     def sub_path(self, path):
         artifact = deepcopy(self)
+        artifact.parent = self
         if artifact._sub_path is None:
             artifact._sub_path = path
         else:
@@ -663,8 +669,11 @@ class InputArtifact(ArgoVar):
             if self.source is None:
                 return V1alpha1Parameter(name="dflow_art_%s" % self.name)
             elif isinstance(self.source, CustomArtifact):
+                if self.source.redirect is not None:
+                    return V1alpha1Parameter(name="dflow_art_%s" % self.name,
+                                             value=self.source.redirect)
                 return V1alpha1Parameter(name="dflow_art_%s" % self.name,
-                                         value=jsonpickle.dumps(self.source))
+                                         value=self.source.get_urn())
             else:
                 return V1alpha1Parameter(name="dflow_art_%s" % self.name,
                                          value=str(self.source))
@@ -1029,6 +1038,7 @@ class OutputArtifact(ArgoVar):
 
     def sub_path(self, path):
         artifact = deepcopy(self)
+        artifact.parent = self
         if artifact._sub_path is None:
             artifact._sub_path = path
         else:
