@@ -12,6 +12,7 @@ from ..executor import Executor, render_script_with_tmp_root, run_script
 from ..io import InputArtifact, InputParameter
 from ..op_template import ScriptOPTemplate
 from ..python import PythonOPTemplate
+from ..python.python_op_template import handle_packages_script
 from ..utils import randstr, upload_s3
 from . import bohrium
 
@@ -158,14 +159,19 @@ class DispatcherExecutor(Executor):
         if self.machine_dict["context_type"] == "Bohrium":
             if "batch_type" not in self.machine_dict:
                 self.machine_dict["batch_type"] = "Bohrium"
-            if "email" not in self.machine_dict["remote_profile"]:
+            self.bohrium_ticket = bohrium.config["ticket"]
+            if self.bohrium_ticket is not None:
+                self.machine_dict["remote_profile"]["email"] = ""
+            elif "email" not in self.machine_dict["remote_profile"]:
                 if bohrium.config["username"] is not None:
                     self.machine_dict["remote_profile"]["email"] = \
                         bohrium.config["username"]
                 else:
                     self.machine_dict["remote_profile"]["email"] = input(
                         "Bohrium email: ")
-            if "password" not in self.machine_dict["remote_profile"]:
+            if self.bohrium_ticket is not None:
+                self.machine_dict["remote_profile"]["password"] = ""
+            elif "password" not in self.machine_dict["remote_profile"]:
                 if bohrium.config["password"] is not None:
                     self.machine_dict["remote_profile"]["password"] = \
                         bohrium.config["password"]
@@ -282,7 +288,12 @@ class DispatcherExecutor(Executor):
                                    if m.name != "launching"]
             new_template.sidecars = [s for s in new_template.sidecars
                                      if not s.name.startswith("rclone-")]
-
+            new_template.script += handle_packages_script(
+                "./tmp/inputs/artifacts/dflow_python_packages")
+            new_template.script += "import jsonpickle\n"
+            new_template.script += "from dflow import config\n"
+            new_template.script += "config['artifact_register'] = %s\n" % \
+                json.dumps(config["artifact_register"])
         new_template.script += "with open('script', 'w') as f:\n"
         new_template.script += "    f.write(r\"\"\"\n"
         if self.map_tmp_dir:
@@ -295,6 +306,10 @@ class DispatcherExecutor(Executor):
         new_template.script += "\"\"\")\n"
 
         if self.machine_dict["context_type"] == "Bohrium":
+            if self.bohrium_ticket is not None:
+                if new_template.envs is None:
+                    new_template.envs = {}
+                new_template.envs["BOHR_TICKET"] = self.bohrium_ticket
             if "image_name" not in self.machine_dict["remote_profile"][
                     "input_data"]:
                 self.machine_dict["remote_profile"]["input_data"][
@@ -309,9 +324,14 @@ class DispatcherExecutor(Executor):
         for name, art in template.inputs.artifacts.items():
             if self.machine_dict["context_type"] == "Bohrium" and \
                     art.save_as_parameter:
+                new_template.script += "import jsonpickle\n"
+                new_template.script += "from dflow import CustomArtifact\n"
+                new_template.script += "bohrium_urn = CustomArtifact.from_urn"\
+                    "('{{inputs.parameters.dflow_art_%s}}').get_bohrium_urn("\
+                    "'%s')\n" % (name, name)
                 new_template.script += "machine.input_data['job_resources']"\
-                    " = machine.input_data.get('job_resources', []) + ['%s']"\
-                    "\n" % art.source.get_bohrium_urn(name)
+                    " = machine.input_data.get('job_resources', []) + "\
+                    "[bohrium_urn]\n"
 
         new_template.script += "resources = Resources.load_from_dict(json."\
             "loads(r'%s'))\n" % json.dumps(self.resources_dict)
