@@ -6,6 +6,8 @@ import json
 import logging
 import os
 from abc import ABC
+from copy import deepcopy
+from functools import partial
 from pathlib import Path
 from typing import Dict, List, Set, Union
 
@@ -13,7 +15,8 @@ from typeguard import check_type
 
 from ..argo_objects import ArgoObjectDict
 from ..config import config
-from ..utils import get_key, s3_config
+from ..context_syntax import GLOBAL_CONTEXT
+from ..utils import get_key, randstr, s3_config
 from .opio import (OPIO, Artifact, BigParameter, OPIOSign, Parameter,
                    type_to_str)
 
@@ -181,7 +184,9 @@ class OP(ABC):
                 check_type(ii, io, ss)
 
     @classmethod
-    def function(cls, func):
+    def function(cls, func=None, **kwargs):
+        if func is None:
+            return partial(cls.function, **kwargs)
         signature = func.__annotations__
         return_type = signature.get('return', None)
 
@@ -206,6 +211,8 @@ class OP(ABC):
                 f'got {type(return_type)}.')
 
         class subclass(cls):
+            task_kwargs = {}
+
             @classmethod
             def get_input_sign(cls):
                 return input_sign
@@ -219,7 +226,29 @@ class OP(ABC):
                 op_out = func(**op_in)
                 return op_out
 
+            def use(self, **kwargs):
+                op = deepcopy(self)
+                op.task_kwargs = kwargs
+                return op
+
             def __call__(self, **op_in):
+                if GLOBAL_CONTEXT.in_context:
+                    from .python_op_template import PythonOPTemplate
+                    from ..task import Task
+                    input_sign = self.get_input_sign()
+                    parameters = {k: v for k, v in op_in.items()
+                                  if not isinstance(input_sign[k], Artifact)}
+                    artifacts = {k: v for k, v in op_in.items()
+                                 if isinstance(input_sign[k], Artifact)}
+                    name = func.__name__.lower().replace("_", "-") + "-" + \
+                        randstr()
+                    task = Task(name,
+                                template=PythonOPTemplate(self, **kwargs),
+                                parameters=parameters, artifacts=artifacts,
+                                **self.task_kwargs)
+                    op_out = {**task.outputs.parameters,
+                              **task.outputs.artifacts}
+                    return op_out
                 return self.execute(op_in)
 
         subclass.func = func
