@@ -89,13 +89,18 @@ def download_artifact(
         skip_exists: skip files with the same MD5
     """
     if config["mode"] == "debug" and not debug_download:
-        linktree(artifact.local_path, path)
-        return assemble_path_list(path, remove=remove_catalog)
+        if config["debug_copy_method"] == "symlink":
+            linktree(artifact.local_path, path)
+        elif config["debug_copy_method"] == "link":
+            merge_dir(artifact.local_path, path, try_link)
+        elif config["debug_copy_method"] == "copy":
+            merge_dir(artifact.local_path, path, shutil.copy)
+        return assemble_path_object(path, remove=remove_catalog)
 
     key = get_key(artifact)
 
     if slice is not None:
-        sub_path = path_list_of_artifact(artifact)[slice]
+        sub_path = path_object_of_artifact(artifact)[slice]
 
     if sub_path is not None:
         key = key + "/" + sub_path
@@ -103,7 +108,8 @@ def download_artifact(
         download_s3(key=key, recursive=True, path=path, keep_dir=True,
                     **kwargs)
         path = os.path.join(path, os.path.basename(sub_path))
-        remove_empty_dir_tag(path)
+        if config["detect_empty_dir"]:
+            remove_empty_dir_tag(path)
         return path
 
     path = download_s3(key=key, recursive=True, path=path, **kwargs)
@@ -125,8 +131,9 @@ def download_artifact(
             else:
                 merge_dir(tmpdir, path)
 
-    remove_empty_dir_tag(path)
-    return assemble_path_list(path, remove=remove_catalog)
+    if config["detect_empty_dir"]:
+        remove_empty_dir_tag(path)
+    return assemble_path_object(path, remove=remove_catalog)
 
 
 def flatten(d: Union[list, dict]) -> dict:
@@ -206,7 +213,8 @@ def upload_artifact(
         archive = config["archive_mode"]
     cwd = os.getcwd()
     with tempfile.TemporaryDirectory() as tmpdir:
-        if isinstance(path, dict):
+        if isinstance(path, dict) or (isinstance(path, list) and any(
+                [isinstance(p, (list, dict)) for p in path])):
             pairs = flatten(path).items()
         elif isinstance(path, (list, set)):
             pairs = enumerate(path)
@@ -231,7 +239,13 @@ def upload_artifact(
                     relpath = abspath[abspath.find(":")+2:]
             target = os.path.join(tmpdir, relpath)
             os.makedirs(os.path.dirname(target), exist_ok=True)
-            os.symlink(abspath, target)
+            if config["mode"] == "debug" and cwd.startswith(p):
+                # To avoid recursive symlink
+                copy_file(abspath, target)
+            else:
+                if not (os.path.exists(target) and
+                        os.path.samefile(abspath, target)):
+                    os.symlink(abspath, target)
             path_list.append({"dflow_list_item": relpath.replace("\\", "/"),
                               "order": i})
 
@@ -444,7 +458,9 @@ def catalog_of_artifact(art, **kwargs) -> List[dict]:
             fname = obj[len(prefix):]
             client.download(key=obj, path=os.path.join(tmpdir, fname))
             with open(os.path.join(tmpdir, fname), "r") as f:
-                catalog += jsonpickle.loads(f.read())['path_list']
+                for item in jsonpickle.loads(f.read())['path_list']:
+                    if item not in catalog:
+                        catalog.append(item)  # remove duplicate
     return catalog
 
 
@@ -452,6 +468,11 @@ def path_list_of_artifact(art, **kwargs) -> List[str]:
     return convert_dflow_list(catalog_of_artifact(art, **kwargs))
 
 
+def path_object_of_artifact(art, **kwargs) -> Union[list, dict]:
+    return assemble_path_object_from_catalog(
+        catalog_of_artifact(art, **kwargs))
+
+
 def force_move(src, dst):
     if os.path.exists(dst):
         if os.path.samefile(src, dst):
@@ -475,7 +496,24 @@ def merge_dir(src, dst, func=force_move):
             func(src_file, dst_file)
 
 
-def copy_file(src, dst, func=os.link):
+def try_link(src, dst):
+    try:
+        if os.path.isfile(dst):
+            os.remove(dst)
+        link(src, dst)
+    except Exception:
+        shutil.copy(src, dst)
+
+
+def link(src, dst):
+    # follow_symlinks=True of os.link not work on Linux
+    if os.path.islink(src):
+        os.link(os.path.realpath(src), dst)
+    else:
+        os.link(src, dst)
+
+
+def copy_file(src, dst, func=try_link):
     os.makedirs(os.path.abspath(os.path.dirname(dst)), exist_ok=True)
     if os.path.isdir(src):
         try:
@@ -488,24 +526,39 @@ def copy_file(src, dst, func=os.link):
         raise RuntimeError("File %s not found" % src)
 
 
-def assemble_path_list(art_path, remove=False):
-    path_list = []
+def catalog_of_local_artifact(art_path, remove=False):
+    catalog = []
     if os.path.isdir(art_path):
-        dflow_list = []
         catalog_dir = os.path.join(art_path, config["catalog_dir_name"])
         if os.path.exists(catalog_dir):
             for f in os.listdir(catalog_dir):
                 with open(os.path.join(catalog_dir, f), 'r') as fd:
                     for item in jsonpickle.loads(fd.read())['path_list']:
-                        if item not in dflow_list:
-                            dflow_list.append(item)  # remove duplicate
+                        if item not in catalog:
+                            catalog.append(item)  # remove duplicate
             if remove:
                 shutil.rmtree(catalog_dir)
-        if len(dflow_list) > 0:
-            path_list = list(map(lambda x: os.path.join(
-                art_path, x) if x is not None else None,
-                convert_dflow_list(dflow_list)))
-    return path_list
+    return catalog
+
+
+def assemble_path_object_from_catalog(catalog, art_path=None):
+    if len(catalog) == 0:
+        return catalog
+    elif all([isinstance(x["order"], int) for x in catalog]):
+        # old fashion
+        return [os.path.join(art_path, x) if art_path is not None and
+                x is not None else x for x in convert_dflow_list(catalog)]
+    else:
+        # new fashion
+        path_dict = {item["order"]: item["dflow_list_item"]
+                     for item in catalog}
+        return expand({k: os.path.join(art_path, v) if art_path is not None and
+                       v is not None else v for k, v in path_dict.items()})
+
+
+def assemble_path_object(art_path, remove=False):
+    catalog = catalog_of_local_artifact(art_path, remove=remove)
+    return assemble_path_object_from_catalog(catalog, art_path=art_path)
 
 
 def convert_dflow_list(dflow_list):
@@ -513,33 +566,6 @@ def convert_dflow_list(dflow_list):
     return list(map(lambda x: x['dflow_list_item'], dflow_list))
 
 
-def assemble_path_dict(art_path, remove=False):
-    path_dict = {}
-    if os.path.isdir(art_path):
-        dflow_list = []
-        catalog_dir = os.path.join(art_path, config["catalog_dir_name"])
-        if os.path.exists(catalog_dir):
-            for f in os.listdir(catalog_dir):
-                with open(os.path.join(catalog_dir, f), 'r') as fd:
-                    for item in jsonpickle.loads(fd.read())['path_list']:
-                        if item not in dflow_list:
-                            dflow_list.append(item)  # remove duplicate
-            if remove:
-                shutil.rmtree(catalog_dir)
-        if len(dflow_list) > 0:
-            for item in dflow_list:
-                if item["dflow_list_item"] is None:
-                    path_dict[item["order"]] = None
-                else:
-                    path_dict[item["order"]] = os.path.join(
-                        art_path, item["dflow_list_item"])
-    return path_dict
-
-
-def assemble_path_nested_dict(art_path, remove=False):
-    return expand(assemble_path_dict(art_path, remove))
-
-
 def remove_empty_dir_tag(path):
     for dn, ds, fs in os.walk(path, followlinks=True):
         if ".empty_dir" in fs:
@@ -779,9 +805,3 @@ class MinioClient(StorageClient):
     def get_md5(self, key: str) -> str:
         return self.client.stat_object(bucket_name=self.bucket_name,
                                        object_name=key).etag
-
-
-def add_prefix_to_slice(prefix, slices):
-    return "(lambda x: ['%s.' + str(i) for i in x] "\
-        "if isinstance(x, list) else '%s.' + str(x))(%s)" % (
-            prefix, prefix, slices)
