@@ -1,6 +1,7 @@
 import json
 import logging
 import os
+import signal
 import sys
 import time
 from copy import deepcopy
@@ -9,6 +10,7 @@ from typing import Any, Dict, List, Optional, Union
 import jsonpickle
 
 from .argo_objects import ArgoStep, ArgoWorkflow
+from .common import subdomain_errmsg, subdomain_regex
 from .config import config, s3_config
 from .context import Context
 from .context_syntax import GLOBAL_CONTEXT
@@ -143,6 +145,8 @@ class Workflow:
         if uid is None:
             uid = id
         self.uid = uid
+        assert subdomain_regex.match(name), "Invalid workflow name '%s': %s"\
+            % (name, subdomain_errmsg)
         self.name = name
         if steps is not None:
             assert isinstance(steps, Steps)
@@ -151,7 +155,7 @@ class Workflow:
             assert isinstance(dag, DAG)
             self.entrypoint = dag
         else:
-            self.entrypoint = Steps(self.name + "-steps")
+            self.entrypoint = None
         self.templates = {}
         self.argo_templates = {}
         self.pvcs = {}
@@ -176,6 +180,7 @@ class Workflow:
 
     def __exit__(self, exc_type, exc_val, exc_tb) -> None:
         GLOBAL_CONTEXT.in_context = False
+        self.submit()
 
     def add(
             self,
@@ -188,6 +193,12 @@ class Workflow:
             step: a step or a list of parallel steps to be added to the
             entrypoint of the workflow
         """
+        if self.entrypoint is None:
+            if isinstance(step, Task) or (isinstance(step, list) and all(
+                    [isinstance(s, Task) for s in step])):
+                self.entrypoint = DAG(self.name + "-dag")
+            else:
+                self.entrypoint = Steps(self.name + "-steps")
         self.entrypoint.add(step)
 
     def submit(
@@ -285,12 +296,15 @@ class Workflow:
             if config["detach"]:
                 pid = os.fork()
                 if pid != 0:
+                    print("Workflow process ID: %s" % pid)
                     os.chdir(cwd)
                     return ArgoWorkflow({"id": self.id})
                 flog = open(os.path.join(wfdir, "log.txt"), "w")
                 os.dup2(flog.fileno(), sys.stdout.fileno())
                 os.dup2(flog.fileno(), sys.stderr.fileno())
             try:
+                with open(os.path.join(wfdir, "pid"), "w") as f:
+                    f.write(str(os.getpid()))
                 entrypoint = deepcopy(self.entrypoint)
                 entrypoint.orig_template = self.entrypoint
                 entrypoint.run(self.id, self.context)
@@ -341,10 +355,12 @@ class Workflow:
                         outputs["parameters"].append(par.recover())
             if hasattr(step.outputs, "artifacts"):
                 for name, art in step.outputs.artifacts.items():
-                    if hasattr(step, "inputs") and \
-                        hasattr(step.inputs, "parameters") and \
-                        "dflow_group_key" in step.inputs.parameters \
-                            and name != "main-logs":
+                    group_key = step.get("inputs", {}).get(
+                        "parameters", {}).get("dflow_group_key", {}).get(
+                        "value")
+                    art_key = get_key(art, raise_error=False)
+                    if group_key and art_key and art_key.endswith(
+                            "%s/%s" % (group_key, name)):
                         if config["overwrite_reused_artifact"]:
                             self.handle_reused_artifact(step, name, art)
                         else:
@@ -419,6 +435,7 @@ class Workflow:
             self.copied_keys.append(old_key)
 
     def convert_to_argo(self, reuse_step=None):
+        self.parents = {}
         if self.context is not None:
             assert isinstance(self.context, (Context, Executor))
             self = self.context.render(self)
@@ -503,6 +520,7 @@ class Workflow:
             workflow_urn = config["lineage"].register_workflow(self.name)
             self.parameters["dflow_workflow_urn"] = workflow_urn
 
+        self.deduplicate_templates()
         return V1alpha1Workflow(
             metadata=metadata,
             spec=V1alpha1WorkflowSpec(
@@ -524,6 +542,44 @@ class Workflow:
             ),
             status=status)
 
+    def deduplicate_templates(self):
+        logger.debug("before deduplication: %s" % len(self.argo_templates))
+        modified = self.argo_templates
+        deduplicated = {}
+        while modified:
+            modified_name = set()
+            for n1, t1 in modified.items():
+                duplicate = False
+                for n2, t2 in deduplicated.items():
+                    t1.name = n2
+                    if t1 == t2:
+                        duplicate = True
+                        logger.debug("template %s == %s, remove %s" % (
+                            n1, n2, n1))
+                        for parent in self.parents.get(n1, []):
+                            if parent.steps:
+                                for step in parent.steps:
+                                    for ps in step:
+                                        if ps.template == n1:
+                                            ps.template = n2
+                            elif parent.dag:
+                                for task in parent.dag.tasks:
+                                    if task.template == n1:
+                                        task.template = n2
+                            self.parents[n2] = self.parents.get(n2, []) + [
+                                parent]
+                            modified_name.add(parent.name)
+                        break
+                if not duplicate:
+                    t1.name = n1
+                    deduplicated[n1] = t1
+            self.argo_templates = deduplicated
+            modified = {k: v for k, v in self.argo_templates.items()
+                        if k in modified_name}
+            deduplicated = {k: v for k, v in self.argo_templates.items()
+                            if k not in modified_name}
+        logger.debug("after deduplication: %s" % len(self.argo_templates))
+
     def to_dict(self):
         return self.api_instance.api_client.sanitize_for_serialization(
             self.convert_to_argo())
@@ -644,9 +700,10 @@ class Workflow:
                 argo_template, templates = template.convert_to_argo(
                     memoize_prefix, memoize_configmap, self.context)
                 self.argo_templates[template.name] = argo_template
-                for template in templates:
-                    self.handle_template(
-                        template, memoize_prefix, memoize_configmap)
+                for t in templates:
+                    self.parents[t.name] = self.parents.get(t.name, []) + [
+                        argo_template]
+                    self.handle_template(t, memoize_prefix, memoize_configmap)
             else:
                 self.argo_templates[template.name] = template.convert_to_argo(
                     memoize_prefix, memoize_configmap)
@@ -741,7 +798,11 @@ class Workflow:
                 stepdir = os.path.join(self.id, s)
                 if not os.path.isdir(stepdir):
                     continue
-                if name is not None and not s.startswith(self.id + "-" + name):
+                if not os.path.exists(os.path.join(stepdir, "name")):
+                    continue
+                with open(os.path.join(stepdir, "name"), "r") as f:
+                    _name = f.read()
+                if name is not None and name != _name:
                     continue
                 if key is not None and s not in key:
                     continue
@@ -760,7 +821,7 @@ class Workflow:
                     continue
                 step = {
                     "workflow": self.id,
-                    "displayName": s,
+                    "displayName": _name,
                     "key": s,
                     "startedAt": os.path.getmtime(stepdir),
                     "phase": _phase,
@@ -906,6 +967,11 @@ class Workflow:
         """
         if self.id is None:
             raise RuntimeError("Workflow ID is None")
+        if config["mode"] == "debug":
+            with open(os.path.join(self.id, "pid"), "r") as f:
+                pid = int(f.read())
+            os.killpg(os.getpgid(pid), signal.SIGTERM)
+            return
         self.api_instance.api_client.call_api(
             '/api/v1/workflows/%s/%s/terminate' % (self.namespace, self.id),
             'PUT', header_params=config["http_headers"])
